-------------------------------------------------------------------------------
-- testbench - Module
-------------------------------------------------------------------------------
--**************************************************************************
-- 
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--
--     (c) Copyright 2003 Xilinx, Inc.
--     All rights reserved.
--
--**************************************************************************
-------------------------------------------------------------------------------
-- Filename:        testbench.vhd
-------------------------------------------------------------------------------
-- Description:
--      Testbench for 64-bit Error Detection and Correction
--
-------------------------------------------------------------------------------
--Structure:
--      --testbench.vhd
--
-------------------------------------------------------------------------------
-- Owners:          Simon Tam 
-- Revision 1.0
--
--     Modification History:                                                   
--     Date     Init          Description                                
--   --------  ------ ---------------------------------------------------------
--   2/26/2003   STAM     Initial Release 
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
USE ieee.std_logic_unsigned.ALL;
use std.textio.all;

ENTITY testbench IS
--  generic ( Tclk : 1 ns );
END testbench;

ARCHITECTURE behavior OF testbench IS 

-- Component Declaration
Component EDC
	PORT(	clk : in std_logic;
       		resetN : in std_logic;
		encin : in std_logic_vector (63 downto 0);
		encout : out std_logic_vector (63 downto 0);
		parity_out : out std_logic_vector (7 downto 0);
     		decin : in std_logic_vector(63 downto 0);
       		decout : out std_logic_vector(63 downto 0);
		parity_in : in std_logic_vector (7 downto 0);
		force_error : in std_logic_vector (1 downto 0);
     		error : out std_logic_vector (1 downto 0)
		);
END COMPONENT;

	CONSTANT tclk : time := 10 ns;
	SIGNAL	clk, reset, load :  std_logic;
	SIGNAL	count, decin, encout, decout:  std_logic_vector (63 downto 0);
	SIGNAL 	encout_1, encout_2, encout_3, encout_4:  std_logic_vector (63 downto 0);
	SIGNAL	parity_out :  std_logic_vector (7 downto 0);	
	signal 	force_error : std_logic_vector (1 downto 0);
	SIGNAL	error :  std_logic_vector (1 downto 0);

BEGIN

-- Component Instantiation
uut: EDC PORT MAP(
                        clk => clk ,
                        resetN => reset, 
			encin => count,
			encout => encout,
			parity_out => parity_out,
         			decin => decin,
         			decout => decout,
			parity_in => parity_out,
			force_error => force_error,
           			error => error
			);

-- initial reset
reset <= '0', '1' after 3.5 * Tclk;

-- encoder output wrapped to decoder input
decin <= encout;

------------------------------
-- clock driver
-------------------------------
clock_driver : process is
	  begin
	    clk <= '0';
	    wait for Tclk;
	    loop
	      clk <= '1', '0' after Tclk / 2;
	      wait for Tclk;
	    end loop;
end process clock_driver;

--  Test Bench Statements
-- main counter
counter: process (CLK, RESET)is
begin
   if RESET='0' then
      COUNT <= (others => '0');
--			count <= 0;
   elsif CLK='1' and CLK'event then
       COUNT <= COUNT + 1;
   end if;
end process;
----------------------------------------
-- data generator

process(clk, reset, count)
  begin
   if RESET='0' then
			force_error <= "00";
			
   elsif CLK='1' and CLK'event then
   case count is 
-- feedthru
      when X"0000000000000000" =>	 force_error <= "00";	
			report LF & "--- Data feedthru ---";   
-- 1 bit error walk-thru
      when X"0000000000000047" =>	 force_error <= "01";	
			report LF & "--- Starting single-bit error test ---";
-- 2-bit error walk-thru 				 		
      when X"000000000000008E" => force_error <= "10";	
			report LF & "--- Single-bit error test completed ---";
			report LF & "--- Starting double-bit error test ---";	
-- 3-bit error walk-thru
      when X"00000000000000D5" =>  force_error <= "11";
			report LF & "--- Double-bit error test completed ---";
			report LF & "--- Starting triple-bit error test --- ";	
      when others =>  
         			force_error <= force_error;	
   end case;
end if;
end process;

-- Verify the original data against the output of the decoder

error_check: process (CLK, RESET)is
begin
   if RESET='0' then
			encout_1 <= (others => '0');
			encout_2 <= (others => '0');
			encout_3 <= (others => '0');			
			encout_4 <= (others => '0');				
   elsif CLK='1' and CLK'event then	
       			encout_1 <= count;	
			encout_2 <= encout_1;
			encout_3 <= encout_2;
			encout_4 <= encout_3;
	if force_error = "01" or force_error = "00" then
			assert decout = encout_4
--			assert error = "0010"
			report "== data comparision error! ==" & LF
			severity failure;
	end if;							
   end if;
end process;

--  End Test Bench 
END;
